@@ -153,6 +153,14 @@
 
 
 /* GCC attributes.  */
+#ifndef GNUPG_GCC_ATTR_FORMAT_ARG
+#if __GNUC__ >= 3 /* Actually 2.8 but testing the major is easier.  */
+# define GNUPG_GCC_ATTR_FORMAT_ARG(a)  __attribute__ ((__format_arg__ (a)))
+#else
+# define GNUPG_GCC_ATTR_FORMAT_ARG(a)
+#endif
+#endif
+
 #if __GNUC__ >= 4
 # define GNUPG_GCC_A_SENTINEL(a) __attribute__ ((sentinel(a)))
 #else
@@ -370,6 +378,9 @@ char *xtryasprintf (const char *fmt, ...) GPGRT_GCC_A_PRINTF(1,2);
 /* Replacement for gcry_cipher_algo_name.  */
 const char *gnupg_cipher_algo_name (int algo);
 
+void obsolete_option (const char *configname, unsigned int configlineno,
+                      const char *name);
+
 const char *print_fname_stdout (const char *s);
 const char *print_fname_stdin (const char *s);
 void print_utf8_buffer3 (estream_t fp, const void *p, size_t n,
@@ -386,6 +397,14 @@ int match_multistr (const char *multistr,const char *match);
 
 int gnupg_compare_version (const char *a, const char *b);
 
+struct debug_flags_s
+{
+  unsigned int flag;
+  const char *name;
+};
+int parse_debug_flag (const char *string, unsigned int *debugvar,
+                      const struct debug_flags_s *flags);
+
 
 /*-- Simple replacement functions. */
 
